/*
Copyright (C) 2001 Chao-Kuo Lin

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#ifndef _CK_CONTROL_H_
#define _CK_CONTROL_H_

#include "ckDialog.h"
#include "ckControlImp.h"
#include "ckControlGDI.h"

class ckControlTable;
class ckValueWrapper;
class ckBaseScanner;
class ckPainter;
enum ckError;

//
// ckChildFactory
//
template <class T, class P>
class ckChildFactory
{
    static auto_ptr<P> Make()
    {
        return auto_ptr<P>(new T);
    }
};


//
// case insensitive search for strict weak ordering
//
struct ckSWO
{
    bool operator()(string const &strl, string const &strr) const
    {
        return stricmp(strl.c_str(), strr.c_str()) < 0;
    }
};



//
// ckControl
//
class ckControl
{
    friend class ckControlSingleLevelIterator;

protected:
    typedef map<string, string, ckSWO>  ckControlMapT;
    typedef ckControlMapT::iterator     ckControlItT;

public:
    typedef struct 
    {
        char *_name;
        DWORD _style;
    } ControlStyleT;

    virtual ~ckControl();

    //
    //  Identity's functions
    //
    string getType()         
    { return getAttribute(TYPE_ATTRIB);}
    virtual string getClassName()   { return getType();}

    void addChild(auto_ptr<ckControl> &child);
    ckError removeChild(ckControl &control);
    ckControl *parent() const { return _parent;}
    virtual auto_ptr<ckControl> Clone();

    //
    // attributes functions
    //
    // do we accept it? in other words, is it not yet defined?
    virtual bool acceptAttribute(string const &attrib)
    {   return !_attributeExists(attrib);    }

    // this function is for adding user defined attrib
    virtual ckError addAttribute(string const &attrib, string const &value);
    virtual ckError addAttribute(string const &attrib, ckValueWrapper &wrapper);

    virtual string getAttribute(string const &attrib);
    virtual ckError setVariable(string const &var, string const &value);
    virtual ckError retreiveVariable(string const &var, string &value);

    //
    // event functions
    //
    virtual ckError addEvent(string const &event, string const &action);
    virtual ckError notify(string const &event);   
    // substitute %[this.blah] to %[rootDialogID.blah]
    virtual ckError eventVariableSubstitution(string const &rootDialogID);

    //
    // control access functions
    //
    // subclass responsible for return _perform<subclassimp>(action, wrapper);
    virtual ckError performAction(string const &action, ckValueWrapper &wrapper) = 0;

    //
    // a createwindowex forwarder to keep our encapsulation
    //
    virtual void createwindow(DWORD xstyle, const char *classname, const char *windowname, DWORD style,
                    int x, int y, int w, int h, HWND parent, HMENU menu, HINSTANCE hinstance);

    HWND getHWND() { return _hwnd;}
    HWND parentHWND() { return (parent() != NULL) ? parent()->getHWND() : NULL; }
    
    virtual DWORD _parseXStyle();
    virtual DWORD _parseStyle();

    virtual void acceptPainter(ckPainter &painter) {}

    //
    // drawing functions
    //
    // this should be changed
    Font *getFont() { return _font.get();}
    Brush *getBG() { return _bgbrush.get();}
    COLORREF *getBGColor() { return _bgcolor.get();}
    COLORREF *getFontColor() { return _fontcolor.get();}
    

    //
    // static functions
    //
    // factory function
    static auto_ptr<ckControl> Create(string const &controltype);

    // windows environment initialization function
    static bool init(HINSTANCE hinst);
    static LRESULT CALLBACK WindowProc(HWND hwnd,	UINT msg,	WPARAM wParam,	LPARAM lParam);
    static HINSTANCE _hInstance;

protected:
    ckControl(string &type);
    virtual ControlStyleT * _getstyle(DWORD &style) = 0;


    //
    // private helpers
    //
    void parent(ckControl *ckparent) { _parent = ckparent; }

    bool _attributeExists(string const &attrib);
    //auto_vector<ckControl>::iterator _findChild(string const &id);


    // private data
    ckControl *_parent;
    ckControl *_root; // topmost dialog control could equal to this
    auto_vector<ckControl>  _children;
    ckControlMapT     _attrib;
    ckControlMapT     _event;
    HWND _hwnd;

    // font, bgcolor, bevellight, beveldark, icon
    auto_ptr<Font> _font;
    auto_ptr<Brush> _bgbrush;
    auto_ptr<COLORREF> _fontcolor;
    auto_ptr<COLORREF> _bgcolor;
};

template<class T>
class ckControlFactory : public ckChildFactory<T, ckControl>
{
public:
    static auto_ptr<ckControl> Make()
    {
        return auto_ptr<ckControl>(new T);
    }
};

#endif
